@@ -116,7 +116,6 @@ static gint       xfce_appfinder_window_sort_items                    (GtkTreeMo
                                                                        GtkTreeIter                 *a,
                                                                        GtkTreeIter                 *b,
                                                                        gpointer                     data);
-static gboolean   xfce_appfinder_should_sort_icon_view                (void);
 static void       xfce_appfinder_window_update_frecency               (XfceAppfinderWindow         *window,
                                                                        GtkTreeModel                *model,
                                                                        const gchar                 *uri);
@@ -139,6 +138,7 @@ struct _XfceAppfinderWindow
 
   GtkTreeModel               *sort_model;
   GtkTreeModel               *filter_model;
+  GtkTreePath                *hover_path;
 
   XfceAppfinderCategoryModel *category_model;
 
@@ -245,6 +245,9 @@ xfce_appfinder_window_init (XfceAppfinderWindow *window)
   if (xfconf_channel_get_bool (window->channel, "/hide-window-decorations", FALSE))
     gtk_window_set_decorated (GTK_WINDOW (window), FALSE);
 
+  if (xfconf_channel_get_bool (window->channel, "/generic-names", FALSE))
+    g_object_set (G_OBJECT (window->model), "generic-names", TRUE, NULL);
+
   vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 6);
   gtk_container_add (GTK_CONTAINER (window), vbox);
   gtk_container_set_border_width (GTK_CONTAINER (vbox), 6);
@@ -258,7 +261,6 @@ xfce_appfinder_window_init (XfceAppfinderWindow *window)
                                                         XFCE_APPFINDER_ICON_SIZE_DEFAULT_ITEM,
                                                         scale_factor);
   window->image = image = gtk_image_new_from_pixbuf (window->icon_find);
-  gtk_style_context_add_class (gtk_widget_get_style_context (GTK_WIDGET (image)), "application-icon");
   gtk_widget_set_size_request (image, 48, 48);
   gtk_widget_set_halign(image, GTK_ALIGN_CENTER);
   gtk_container_add (GTK_CONTAINER (hbox), image);
@@ -445,6 +447,8 @@ xfce_appfinder_window_finalize (GObject *object)
     g_object_unref (G_OBJECT (window->filter_category));
   g_free (window->filter_text);
 
+  gtk_tree_path_free (window->hover_path);
+
   (*G_OBJECT_CLASS (xfce_appfinder_window_parent_class)->finalize) (object);
 }
 
@@ -594,7 +598,7 @@ xfce_appfinder_window_set_item_width (XfceAppfinderWindow *window)
 
     case XFCE_APPFINDER_ICON_SIZE_SMALL:
       padding = 4;
-      width = 36 * 2.5;
+      width = 32 * 2.5;
       break;
 
     case XFCE_APPFINDER_ICON_SIZE_NORMAL:
@@ -687,6 +691,64 @@ xfce_appfinder_window_view_button_press_event (GtkWidget           *widget,
 
       if (have_selection)
         return xfce_appfinder_window_popup_menu (widget, window);
+
+    }
+
+  return FALSE;
+}
+
+
+
+static gboolean
+xfce_appfinder_window_view_motion_notify_event (GtkWidget           *widget,
+                                                GdkEventMotion      *event,
+                                                XfceAppfinderWindow *window)
+{
+  gint         x, y;
+  GtkTreePath *path;
+  GdkCursor   *cursor;
+  gboolean     sc_selection = FALSE;
+
+  if (GTK_IS_TREE_VIEW (widget)
+      && gtk_tree_view_get_activate_on_single_click (GTK_TREE_VIEW (widget)))
+    {
+      gtk_tree_view_convert_widget_to_bin_window_coords (GTK_TREE_VIEW (widget),
+                                                         event->x, event->y, &x, &y);
+
+      if (gtk_tree_view_get_path_at_pos (GTK_TREE_VIEW (widget), x, y, &path, NULL, NULL, NULL))
+        sc_selection = TRUE;
+    }
+  else if (GTK_IS_ICON_VIEW (widget)
+           && gtk_icon_view_get_activate_on_single_click (GTK_ICON_VIEW (widget)))
+    {
+      path = gtk_icon_view_get_path_at_pos (GTK_ICON_VIEW (widget), event->x, event->y);
+      if (path != NULL)
+        sc_selection = TRUE;
+    }
+
+  if (sc_selection)
+    {
+      /* check if hovering over a new selection */
+      if (window->hover_path == NULL
+          || (gtk_tree_path_compare (path, window->hover_path) != 0))
+        {
+          gtk_tree_path_free (window->hover_path);
+          window->hover_path = path;
+
+          /* set to 'single-click' hand cursor */
+          cursor = gdk_cursor_new_for_display (gdk_window_get_display (event->window), GDK_HAND2);
+          gdk_window_set_cursor (event->window, cursor);
+          g_object_unref (cursor);
+        }
+      else
+        gtk_tree_path_free (path);
+    }
+  else if (window->hover_path != NULL)
+    {
+      /* moved off selection, reset cursor */
+      gtk_tree_path_free (window->hover_path);
+      window->hover_path = NULL;
+      gdk_window_set_cursor (event->window, NULL);
     }
 
   return FALSE;
@@ -702,8 +764,11 @@ xfce_appfinder_window_view (XfceAppfinderWindow *window)
   GtkTreeSelection  *selection;
   GtkWidget         *view;
   gboolean           icon_view;
+  gboolean           sc_execute;
 
   icon_view = xfconf_channel_get_bool (window->channel, "/icon-view", FALSE);
+  sc_execute = xfconf_channel_get_bool (window->channel, "/single-click-execute", FALSE);
+
   if (window->view != NULL)
     {
       if (icon_view && GTK_IS_ICON_VIEW (window->view))
@@ -728,10 +793,7 @@ xfce_appfinder_window_view (XfceAppfinderWindow *window)
 
   if (icon_view)
     {
-      window->view = view = gtk_icon_view_new_with_model (
-        xfce_appfinder_should_sort_icon_view () ?
-          window->sort_model :
-          window->filter_model);
+      window->view = view = gtk_icon_view_new_with_model (window->sort_model);
 
       gtk_icon_view_set_selection_mode (GTK_ICON_VIEW (view), GTK_SELECTION_SINGLE);
       gtk_icon_view_set_pixbuf_column (GTK_ICON_VIEW (view), XFCE_APPFINDER_MODEL_COLUMN_PIXBUF);
@@ -740,6 +802,9 @@ xfce_appfinder_window_view (XfceAppfinderWindow *window)
       gtk_icon_view_set_row_spacing (GTK_ICON_VIEW (view), 0);
       xfce_appfinder_window_set_item_width (window);
 
+      if (sc_execute)
+        gtk_icon_view_set_activate_on_single_click (GTK_ICON_VIEW (view), sc_execute);
+
       g_signal_connect_swapped (G_OBJECT (view), "selection-changed",
           G_CALLBACK (xfce_appfinder_window_item_changed), window);
       g_signal_connect_swapped (G_OBJECT (view), "item-activated",
@@ -751,6 +816,10 @@ xfce_appfinder_window_view (XfceAppfinderWindow *window)
       gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (view), FALSE);
       gtk_tree_view_set_enable_search (GTK_TREE_VIEW (view), FALSE);
       gtk_tree_view_set_tooltip_column (GTK_TREE_VIEW (view), XFCE_APPFINDER_MODEL_COLUMN_TOOLTIP);
+
+      if (sc_execute)
+        gtk_tree_view_set_activate_on_single_click (GTK_TREE_VIEW (view), sc_execute);
+
       g_signal_connect_swapped (G_OBJECT (view), "row-activated",
           G_CALLBACK (xfce_appfinder_window_row_activated), window);
       g_signal_connect_swapped (G_OBJECT (view), "start-interactive-search",
@@ -790,6 +859,8 @@ xfce_appfinder_window_view (XfceAppfinderWindow *window)
       G_CALLBACK (xfce_appfinder_window_popup_menu), window);
   g_signal_connect (G_OBJECT (view), "button-press-event",
       G_CALLBACK (xfce_appfinder_window_view_button_press_event), window);
+  g_signal_connect (G_OBJECT (view), "motion-notify-event",
+      G_CALLBACK (xfce_appfinder_window_view_motion_notify_event), window);
   gtk_container_add (GTK_CONTAINER (window->viewscroll), view);
   gtk_widget_show (view);
 }
@@ -1719,6 +1790,19 @@ xfce_appfinder_window_property_changed (XfconfChannel       *channel,
       gboolean hide = g_value_get_boolean (value);
       gtk_window_set_decorated (GTK_WINDOW (window), !hide);
     }
+  else if (g_strcmp0 (prop, "/single-click-execute") == 0)
+    {
+      gboolean sc_execute = g_value_get_boolean (value);
+      if (GTK_IS_TREE_VIEW (window->view))
+        gtk_tree_view_set_activate_on_single_click (GTK_TREE_VIEW (window->view), sc_execute);
+      else
+        gtk_icon_view_set_activate_on_single_click (GTK_ICON_VIEW (window->view), sc_execute);
+    }
+  else if (g_strcmp0 (prop, "/generic-names") == 0)
+    {
+      gboolean generic_names = g_value_get_boolean (value);
+      g_object_set (G_OBJECT (window->model), "generic-names", generic_names, NULL);
+    }
 }
 
 
@@ -1860,17 +1944,10 @@ xfce_appfinder_window_execute_command (const gchar          *text,
 
       if (G_LIKELY (g_shell_parse_argv (expanded, NULL, &argv, error)))
         {
-#if LIBXFCE4UI_CHECK_VERSION (4, 15, 6)
           succeed = xfce_spawn (screen, NULL, argv, envp,
                                 G_SPAWN_SEARCH_PATH, FALSE,
                                 gtk_get_current_event_time (), NULL,
                                 TRUE, error);
-#else
-          succeed = xfce_spawn_on_screen (screen, NULL, argv, envp,
-                                          G_SPAWN_SEARCH_PATH, FALSE,
-                                          gtk_get_current_event_time (), NULL,
-                                          error);
-#endif
         }
 
       g_strfreev (argv);
@@ -1924,7 +2001,7 @@ xfce_appfinder_window_execute (XfceAppfinderWindow *window,
           child_model = model;
           gtk_tree_model_get (model, &iter, XFCE_APPFINDER_MODEL_COLUMN_URI, &uri, -1);
 
-          if (GTK_IS_TREE_MODEL_SORT (model) || xfce_appfinder_should_sort_icon_view ())
+          if (GTK_IS_TREE_MODEL_SORT (model))
             {
               gtk_tree_model_sort_convert_iter_to_child_iter (GTK_TREE_MODEL_SORT (model), &child_iter, &iter);
               iter = child_iter;
@@ -1937,7 +2014,7 @@ xfce_appfinder_window_execute (XfceAppfinderWindow *window,
           xfce_appfinder_window_update_frecency (window, model, uri);
           if (!result && regular_command)
             {
-              gtk_tree_model_get (model, &child_iter, XFCE_APPFINDER_MODEL_COLUMN_COMMAND, &cmd, -1);
+              gtk_tree_model_get (GTK_TREE_MODEL (window->model), &child_iter, XFCE_APPFINDER_MODEL_COLUMN_COMMAND, &cmd, -1);
               result = xfce_appfinder_window_execute_command (cmd, screen, window, FALSE, NULL, &error);
               g_free (cmd);
             }
@@ -2134,20 +2211,6 @@ xfce_appfinder_window_sort_items_frecency  (GtkTreeModel *model,
 
 
 
-/*
- * Checks for gtk => 3.22.27 during runtime.
- * This is necessary because sort model for icon view did not work as expected.
- */
-static gboolean
-xfce_appfinder_should_sort_icon_view (void)
-{
-  return gtk_get_major_version () >= 3  &&
-         gtk_get_minor_version () >= 22 &&
-         gtk_get_micro_version () >= 27;
-}
-
-
-
 static gchar**
 xfce_appfinder_parse_envp (gchar **cmd)
 {
