@@ -48,6 +48,8 @@ static gboolean            opt_version = FALSE;
 static gboolean            opt_replace = FALSE;
 static gboolean            opt_quit = FALSE;
 static gboolean            opt_disable_server = FALSE;
+static gboolean            opt_toggle_window = FALSE;
+static gboolean            opt_daemon = FALSE;
 static GSList             *windows = NULL;
 static gboolean            service_owner = FALSE;
 static XfceAppfinderModel *model_cache = NULL;
@@ -66,6 +68,8 @@ static GOptionEntry option_entries[] =
   { "replace", 'r', 0, G_OPTION_ARG_NONE, &opt_replace, N_("Replace the existing service"), NULL },
   { "quit", 'q', 0, G_OPTION_ARG_NONE, &opt_quit, N_("Quit all instances"), NULL },
   { "disable-server", 0, 0, G_OPTION_ARG_NONE, &opt_disable_server, N_("Do not try to use or become a D-Bus service"), NULL },
+  { "toggle-window", 't', 0, G_OPTION_ARG_NONE, &opt_toggle_window, N_("Toggle window visibility of background service"), NULL },
+  { "daemon", 0, 0, G_OPTION_ARG_NONE, &opt_daemon, N_("Run in daemon mode"), NULL },
   { NULL }
 };
 
@@ -179,8 +183,9 @@ appfinder_window_destroyed (GtkWidget *window)
 
 
 void
-appfinder_window_new (const gchar *startup_id,
-                      gboolean     expanded)
+appfinder_window_open (const gchar            *startup_id,
+                       gboolean                expanded,
+                       XfceAppfinderWindowHint hint)
 {
   GtkWidget *window;
   XfconfChannel *channel;
@@ -191,7 +196,12 @@ appfinder_window_new (const gchar *startup_id,
       if (xfconf_channel_get_bool (channel, "/enable-service", TRUE) &&
           xfconf_channel_get_bool (channel, "/single-window", TRUE))
         {
-          gtk_window_present (GTK_WINDOW (g_slist_nth_data (windows, 0)));
+          window = g_slist_nth_data (windows, 0);
+          if ((hint & XFCE_APPFINDER_WINDOW_HINT_TOGGLE) && gtk_widget_is_visible (window))
+            gtk_window_close (GTK_WINDOW (window));
+          else
+            gtk_window_present (GTK_WINDOW (window));
+
           return;
         }
     }
@@ -201,7 +211,9 @@ appfinder_window_new (const gchar *startup_id,
                          NULL);
   appfinder_refcount_debug_add (G_OBJECT (window), startup_id);
   xfce_appfinder_window_set_expanded (XFCE_APPFINDER_WINDOW (window), expanded);
-  gtk_widget_show (window);
+
+  if (!(hint & XFCE_APPFINDER_WINDOW_HINT_HIDDEN))
+    gtk_widget_show (window);
 
   windows = g_slist_prepend (windows, window);
   g_signal_connect (G_OBJECT (window), "destroy",
@@ -213,9 +225,11 @@ appfinder_window_new (const gchar *startup_id,
 gint
 main (gint argc, gchar **argv)
 {
-  GError      *error = NULL;
-  const gchar *startup_id;
-  GSList      *windows_destroy;
+  GError                            *error = NULL;
+  const gchar                       *startup_id;
+  GSList                            *windows_destroy;
+  XfceAppfinderGdbusAction           action;
+  XfceAppfinderGdbusActionParameters params;
 
   /* set translation domain */
   xfce_textdomain (GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR, "UTF-8");
@@ -251,9 +265,11 @@ main (gint argc, gchar **argv)
     }
 
   if (opt_quit)
-    {
-      return appfinder_gdbus_quit (NULL) ? EXIT_SUCCESS : EXIT_FAILURE;
-    }
+    return appfinder_gdbus_action (XFCE_APPFINDER_GDBUS_ACTION_QUIT, NULL, NULL) ? EXIT_SUCCESS : EXIT_FAILURE;
+  else if (opt_toggle_window)
+    action = XFCE_APPFINDER_GDBUS_ACTION_TOGGLE;
+  else
+    action = XFCE_APPFINDER_GDBUS_ACTION_OPEN;
 
   /* if started with the xfrun4 executable, start in collapsed mode */
   if (!opt_collapsed && strcmp (*argv, "xfrun4") == 0)
@@ -263,15 +279,17 @@ main (gint argc, gchar **argv)
    * owner to spawn an instance */
   if (G_LIKELY (!opt_disable_server))
     {
-      /* try to open a new window */
-      if (appfinder_gdbus_open_window (!opt_collapsed, startup_id, &error))
+      if (!opt_daemon)
         {
-          /* looks ok */
-          return EXIT_SUCCESS;
-        }
-      else if (!g_error_matches (error, G_DBUS_ERROR, G_DBUS_ERROR_NAME_HAS_NO_OWNER))
-        {
-          g_warning ("Unknown DBus error: %s", error->message);
+          /* try to open a new window */
+          params.expanded = (!opt_collapsed);
+          params.startup_id = startup_id;
+
+          if (appfinder_gdbus_action (action, &params, &error))
+            return EXIT_SUCCESS; /* looks ok */
+
+          if (!g_error_matches (error, G_DBUS_ERROR, G_DBUS_ERROR_NAME_HAS_NO_OWNER))
+            g_warning ("Unknown DBus error: %s", error->message);
         }
 
       g_clear_error (&error);
@@ -305,12 +323,23 @@ main (gint argc, gchar **argv)
   appfinder_refcount_debug_init ();
 #endif
 
+  if (opt_toggle_window && opt_disable_server)
+    g_warning ("Ignoring toggle window request, creating new window");
+
   /* create initial window */
-  appfinder_window_new (NULL, !opt_collapsed);
+  appfinder_window_open (NULL, !opt_collapsed,
+                         opt_daemon ? XFCE_APPFINDER_WINDOW_HINT_HIDDEN :
+                                      XFCE_APPFINDER_WINDOW_HINT_NONE);
 
-  APPFINDER_DEBUG ("enter mainloop");
-
-  gtk_main ();
+  if (!service_owner && opt_daemon)
+    {
+      APPFINDER_DEBUG ("skip mainloop (nothing to do)");
+    }
+  else
+    {
+      APPFINDER_DEBUG ("enter mainloop");
+      gtk_main ();
+    }
 
   /* release the model cache */
   if (model_cache != NULL)
